/*
 * Recover.
 *
 * Martin Houston.
 * 36 Whetstone Close.
 * Farquhar Road,
 * Edgbaston,
 * Birmingham. B15 2QN
 *
 * THIS PROGRAM MAY BE FREELY DISTRIBUTED & MODIFIED WITHOUT CHARGE AS
 * LONG AS THE NAME OF THE ORIGINAL AUTHOR (ME) IS DISPLAYED.
 *
 * If using this program saves your bacon then a payment out of gratitude
 * would no go amiss!
 */
/********************************************************************
 * C Users Group (U.K) C Source Code Library File CUGLIB.020        *
 * Inquiries to: M. Houston, 36 Whetstone Clo. Farquhar Rd.         *
 * Edgbaston, Birmingham B15 2QN ENGLAND			    *
 ********************************************************************
 * File name: recover.c
 * Program name: recover
 * Source of file: Martin Houston
 * Purpose: Recovery of blocks from the Xenix filesystem free list
 * to recover a file that has been accidentaly deleted.
 * (See recover.doc for details).
 * Changes: <who what when & why major changes have been made>      
 ********************************************************************/

#include <fcntl.h>
#include <sys/param.h>
#include <sys/filsys.h>

char filsys[BSIZE]; /* super-block */
char buffer[BSIZE]; /* block buffer */

main(argc, argv)
int argc;
char **argv;
{
	char *fsname;
	int numblocks;
	int fd;

	if(argc != 3)
	{
        	printf("%s <filesys> <count of blks to recover>\n", argv[0]);
		exit(-1);
	}
	fsname = argv[1];
	numblocks = atoi(argv[2]);

	if((fd = open(fsname, O_RDONLY)) < 0)
	{
                printf("%s cannot open %s!\n", argv[0], fsname);
		exit(-1);
	}
	
	lseek(fd, (long)(BSIZE), 0);
	if(read(fd, filsys, BSIZE) < BSIZE)
	{
                printf("%s cannot read superblock from %s!\n", argv[0], fsname);
		exit(-1);
	}

	traverse_freelist((struct filsys *)filsys, fd, 0, numblocks);
}

/*
 * Function that writes out a 1k buffer into a numbered disk file.
 */
write_file(buf, num)
char *buf;
int num;
{
	char filename[20];
	int fd;

        /*
         * Creat files in the current directory. 
  	 */
	sprintf(filename, "flist%d", num);
        
	fd = open(filename, O_WRONLY|O_CREAT, 0666);
	write(fd, buf, BSIZE);
	close(fd);
}


traverse_freelist(fsp, fd, start, end)
struct filsys *fsp;
int fd,start,end;
{
	int freenum = 0;
	daddr_t blkno;

        while(start < end)
	{	
		if((freenum = --(fsp->s_nfree)) == 0)
			get_more_free(fsp, fd);
		else
		{
			/* 
    			 * the most recent free block is located at 
			 * s_free[nfree].
			 */
			blkno = fsp->s_free[freenum];
			if(blkno == 0L)
			{
				printf("Bad blocknum!\n");
				exit(-1);
			}
			lseek(fd, (long)(blkno * BSIZE), 0);
			read(fd, buffer, BSIZE);
			printf("File %d for block %ld\n", start, blkno);
			write_file(buffer, start++);
		}
	}
}

get_more_free(fsp, fd)
struct filsys *fsp;
int fd;
{
	daddr_t blkno;
	daddr_t *lb;

	blkno = fsp->s_free[0]; /* next set */
	if(blkno == 0L)
	{
		/*
                 * No more blocks on the freelist!
                 */
		printf("Bad blocknum!\n");
		exit(-1);
	}
	lseek(fd, (long)(blkno * BSIZE), 0);
	read(fd, buffer, BSIZE);
	lb = (daddr_t *)buffer;
	fsp->s_nfree = lb[0];
	memcpy(fsp->s_free, &(lb[1]), NICFREE * sizeof(daddr_t));
}
